const fs = require('fs-extra');
const getAutogeneratedFileWarning = require('./functions/getAutogeneratedFileWarning');
const getFileImports = require('./functions/getFileImports');
const getHotReloadAcceptor = require('./functions/getHotReloadAcceptor');
const getPassageObjectDefinitions = require('./functions/getAuthoredAssetObjectDefinitions');
const path = require('path');
const scrapeAssets = require('./functions/scrapeAssets');

const authoredHeadersDir = path.join(__dirname, '..', 'headers');

(async () => {
  try {
    const files = await scrapeAssets(authoredHeadersDir);

    const {
      importPaths,
      imports,
    } = getFileImports(files);

    const manifestStr =
      getAutogeneratedFileWarning() +
      '\n\nimport { IHeader } from \'../src/passages/IHeader\';\n\n' +
      imports.join('\n') +
      '\nconst manifest: Array<{ filepath: string, headerObject: IHeader, }> = [\n' +
      getPassageObjectDefinitions(files, 'headerObject').join('\n') +
      '\n];\n\nexport default manifest;\n' +
      getHotReloadAcceptor(importPaths) +
      '\n';

    const manifestPath = path.join(authoredHeadersDir, 'headers-manifest.ts');
    await fs.writeFile(manifestPath, manifestStr);
  } catch (err) {
    console.error(err);
    process.exit(1);
  }
})();
