const fs = require('fs-extra');
const getAutogeneratedFileWarning = require('./functions/getAutogeneratedFileWarning');
const getAuthoredAssetObjectDefinitions = require('./functions/getAuthoredAssetObjectDefinitions');
const getFileImports = require('./functions/getFileImports');
const getHotReloadAcceptor = require('./functions/getHotReloadAcceptor');
const path = require('path');
const scrapeAssets = require('./functions/scrapeAssets');

const authoredPluginsDir = path.join(__dirname, '..', 'plugins');

(async () => {
  try {
    const files = await scrapeAssets(authoredPluginsDir);

    const {
      importPaths,
      imports,
    } = getFileImports(files);
  
    const manifestStr =
      getAutogeneratedFileWarning() +
      'import { IPluginExport, } from \'../src/plugins/IPluginExport\';\n' +
      imports.join('\n') +
      '\nconst manifest: Array<{ filepath: string, pluginExport: IPluginExport }> = [\n' +
      getAuthoredAssetObjectDefinitions(files, 'pluginExport').join('\n') +
      '\n];\n\n' +
      'export default manifest;\n' + 
      getHotReloadAcceptor(importPaths) +
      '\n';

    const manifestPath = path.join(authoredPluginsDir, 'plugins-manifest.ts');
    await fs.writeFile(manifestPath, manifestStr);
  } catch (err) {
    console.error(err);
    process.exit(1);
  }
})();
